package simFil;
/*
	FilSegment .... the sections that form a filament
*/

/*  <SimFil - simulation of cytoskeletal filament flexibility by Pairwise Agent Interaction with Rational Superposition (PAIRS)>
    Copyright (C) <2008>  <Jonathan B. Alberts>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import com.sun.j3d.utils.geometry.Cylinder;
import com.sun.j3d.utils.geometry.Primitive;
import com.sun.j3d.utils.geometry.Sphere;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class FilSegment extends Thing {
	static FilSegment [] theFilSegments = new FilSegment [10000];	// array holding all filament segments
	static int filSegmentCt = 0;
	static int filamentRenderCt = 0;	// for rendering only
	static int filSegRenderCt = 0;		// for rendering only
	
	static final Object filSync = new Object();	// for synchronizing filament creation
	
	// filSegments grouped by the filament they belong to... assign each a unique filament number
	static int filCt = 0; 		// tracks number of filaments
	static int filIDCt = 0;	// ever increasing id numbers for filaments
	int filID;			// all segments in the same filament share the same id number
	int filArrayPos;		// position in the static array of filsegments
	
	// if drags are for filaments near plane
	static double cParallel,cPerp,cRot; 
	
	Pt3D end1 = new Pt3D();
	Pt3D end2 = new Pt3D();
	// Pt3Ds for plasmid force calculations
	Pt3D F = new Pt3D();
	Pt3D Fopp = new Pt3D();
	Pt3D R = new Pt3D();
	Pt3D RcrossF = new Pt3D();
	Pt3D toPlasmidUVec = new Pt3D();
	Pt3D torsionVec = new Pt3D();
	
	// info about end states and segments
	boolean filAtEnd1 = false;
	boolean filAtEnd2 = false;
	FilSegment end1Fil = null;
	FilSegment end2Fil = null;
	Pt3D ptAtEnd1;
	Pt3D ptAtEnd2;
	boolean end1LinkCkd = false;
	boolean end2LinkCkd = false;
	boolean end1TorqCkd = false;
	boolean end2TorqCkd = false;
	
	int end2LinkThingNumber;		// used in reading QK state... myThingNumber of end2 filsegment if any
	int end1PlasmidThingNumber;		// ditto
	int end2PlasmidThingNumber;		// ditto
	
	Pt3D linkUVec = new Pt3D();		// a recycled Pt3D for segment link force calculations
	Pt3D linkUVecR = new Pt3D();		// reverse direction of above
	Pt3D linkPt = new Pt3D();			// link point location... recycled
	
	int end1DetachCounter = 0;
	int end2DetachCounter = 0;
	int minusEndDelta = 0;		// keeps track of changes to minus end position
	
	Pt3D monInc=new Pt3D(),monOffset=new Pt3D(),evenStart=new Pt3D(),evenStop=new Pt3D(),oddStart=new Pt3D(),oddStop=new Pt3D();
	double end1AxialF = 0;
	double end2AxialF = 0;
	double end1TipC = 1e6; // large number for initial tip clearance of end1
	double end2TipC = 1e6; // large number for initial tip clearance of end2
	double fturn;
	double ftrans;
	double fnorm;
	Pt3D Fcoll = new Pt3D();	// recycled pt for force calculations
	Pt3D Tcoll = new Pt3D();	// recycled pt for torque calculations
	boolean lengthChanged = false;
	boolean plasmidAtEnd1 = false;
	boolean plasmidAtEnd2 = false;
	static boolean releaseAllPlasmids = false;
	boolean globalPlasAtEnd1 = false;
	boolean globalPlasAtEnd2 = false;
	boolean end1Catastrophy = false;
	boolean end2Catastrophy = false;
	PSphere end1Plasmid,end2Plasmid;
	Pt3D end1PAttachPt = new Pt3D(); Pt3D end2PAttachPt = new Pt3D();
	Pt3D end1PAttachPtInX = new Pt3D(); Pt3D end2PAttachPtInX = new Pt3D();
	Pt3D end1MonCenter = new Pt3D();
	Pt3D end2MonCenter = new Pt3D();
	Pt3D coordMonCenter = new Pt3D();
	Vector3d end1Vec3d = new Vector3d();
	Vector3d end2Vec3d = new Vector3d();
	Vector3d coordVec3d = new Vector3d();
	Pt3D curMonStart = new Pt3D();
	Pt3D curMonStop = new Pt3D();
	
	double length;
	double l;
	double helixAng = 2*Math.PI*Env.mtRNG.raw();	// keeps track of the helix angle of minusMon
	int monomerCt = 0;

	// for graphics
	TransformGroup cylTG,end1TG,end2TG,coordTG;
	Transform3D cylT3D,cylRot,end1T3D,end2T3D,coordT3D;
	BranchGroup cylBG,end1BG,end2BG,coordBG,coordSysG;
	Cylinder myCyl;
	Sphere end1S,end2S,coordS;
	boolean updateGraphicsLength = false;
	static boolean graphicsInitialized = false;
	static float shiny = 128.0f;
	static Color3f ambientC = new Color3f(0.0f,1.0f,0.0f);
	static Color3f diffuseC = new Color3f(0.9f,0.9f,0.9f);
	static Color3f specularC = new Color3f(1.0f,1.0f,1.0f);
	static Color3f emissiveC = new Color3f(0.0f,0.0f,1.0f);
	static Color3f catasphColor = new Color3f(1.0f,0.0f,0.0f);
	static Color3f noPlasmidColor = new Color3f(0.0f,1.0f,1.0f);
	static Color3f gotPlasmidColor = new Color3f(0.0f,1.0f,1.0f);
	static Appearance catasphApp = new Appearance();
	static Appearance noPlasmidApp = new Appearance();
	static Appearance gotPlasmidApp = new Appearance();
	static ColoringAttributes catasphCA = new ColoringAttributes(catasphColor, ColoringAttributes.FASTEST);
	static ColoringAttributes noPlasmidCA = new ColoringAttributes(noPlasmidColor, ColoringAttributes.FASTEST);
	static ColoringAttributes gotPlasmidCA = new ColoringAttributes(gotPlasmidColor, ColoringAttributes.FASTEST);
	static Material catasphMat = new Material(catasphColor,catasphColor,catasphColor,catasphColor,shiny);
	static Material noPlasmidMat = new Material(noPlasmidColor,noPlasmidColor,noPlasmidColor,noPlasmidColor,shiny);
	static Material gotPlasmidMat = new Material(gotPlasmidColor,gotPlasmidColor,gotPlasmidColor,gotPlasmidColor,shiny);
	static TransparencyAttributes tA = new TransparencyAttributes (TransparencyAttributes.FASTEST,0.6f);
	// for coord sys drawing
	static Color3f xLineColor = new Color3f(1.0f,0.0f,0.0f);
	static Color3f yLineColor = new Color3f(0.0f,1.0f,0.0f);
	static Color3f zLineColor = new Color3f(0.0f,0.0f,1.0f);
	static Appearance xLineApp = new Appearance();
	static Appearance yLineApp = new Appearance();
	static Appearance zLineApp = new Appearance();
	static ColoringAttributes xLineCA = new ColoringAttributes(xLineColor, ColoringAttributes.FASTEST);
	static ColoringAttributes yLineCA = new ColoringAttributes(yLineColor, ColoringAttributes.FASTEST);
	static ColoringAttributes zLineCA = new ColoringAttributes(zLineColor, ColoringAttributes.FASTEST);
	double coordLineLength = 5*Env.monomerDiameter.getValue();
	LineArray xLine, yLine, zLine;
	Shape3D xLineShape, yLineShape, zLineShape;
	Pt3D xLineEndPt = new Pt3D();
	Pt3D yLineEndPt = new Pt3D();
	Pt3D zLineEndPt = new Pt3D();
	boolean coordSysOn = false;
	boolean end1BGOn = false;
	boolean end2BGOn = false;

	public FilSegment (Pt3D initCoord, Pt3D initUVec, int filID, int monomerCt, boolean fromFile) {
		super(initCoord);
		synchronized (filSync) {
			uVec.copy(initUVec);
			yVec = Pt3D.RandomUnitVec();
			this.monomerCt = monomerCt;
			length = (monomerCt+1)*Env.monomerDiameter.getValue()/2;
			addFilSegment(this);
			setFilamentID(this, filID);
			calculateProperties();
			initialize();
		}
	}
	
	public FilSegment (Pt3D initCoord, Pt3D initUVec, int monomerCt, FilSegment splitFromFil) {
		super(initCoord);
		synchronized (filSync) {
			uVec.copy(initUVec);
			yVec = Pt3D.RandomUnitVec();
			this.monomerCt = monomerCt;
			length = (monomerCt+1)*Env.monomerDiameter.getValue()/2;
			addFilSegment(this);
			setFilamentID(this, splitFromFil.filID);
			globalPlasAtEnd1 = splitFromFil.globalPlasAtEnd1;
			globalPlasAtEnd2 = splitFromFil.globalPlasAtEnd2;
			calculateProperties();
			initialize();
			setEnd1Links(splitFromFil, true);
			if (splitFromFil.filAtEnd2) {
				if (splitFromFil.ptAtEnd2 == splitFromFil.end2Fil.end1) { // same orientation
					setEnd2Links(splitFromFil.end2Fil, true);
					splitFromFil.end2Fil.setEnd1Links(this, true);
				} else {
					setEnd2Links(splitFromFil.end2Fil, false);
					splitFromFil.end2Fil.setEnd2Links(this, false);
				}
			}
			
			if (splitFromFil.plasmidAtEnd2) { 				// transfer plasmid link
				transferEnd2Plasmid(splitFromFil,this);
			}
		}
	}
	
	public void sepaku () {
		super.sepaku();
		end1 = null;
		end2 = null;
		F = null;
		Fopp = null;
		R = null;
		RcrossF = null;
		toPlasmidUVec = null;
		torsionVec = null;
		
		end1Fil = null;
		end2Fil = null;
		ptAtEnd1 = null;
		ptAtEnd2 = null;
		
		linkUVec = null;
		linkUVecR = null;
		linkPt = null;
		
		monInc=null;
		monOffset=null;
		evenStart=null;
		evenStop=null;
		oddStart=null;
		oddStop=null;
		
		Fcoll = null;
		Tcoll = null;
		
		end1Plasmid = null;
		end2Plasmid = null;
		end1PAttachPt = null; 
		end2PAttachPt = null;
		end1PAttachPtInX = null; 
		end2PAttachPtInX = null;
		end1MonCenter = null;
		end2MonCenter = null;
		coordMonCenter = null;
		end1Vec3d = null;
		end2Vec3d = null;
		coordVec3d = null;
		curMonStart = null;
		curMonStop = null;
		
		cylTG=null; end1TG=null; end2TG=null; coordTG=null;
		cylT3D=null; end1T3D=null; end2T3D=null; coordT3D=null;
		end1BG=null; end2BG=null; coordBG=null; coordSysG=null;
		myCyl=null;
		end1S=null; end2S=null; coordS=null;
	
		xLine=null; yLine=null; zLine=null;
		xLineShape=null; yLineShape=null; zLineShape=null;
		xLineEndPt = null;
		yLineEndPt = null;
		zLineEndPt = null;
	}
	
	public void calculateProperties () {
		calculateDragTerms();
		// define the constants for motion of this rod in viscous medium
		// Remember that the dimensions we've been using are in micrometers so....
		// **WARNING** below a certain number of monomers, depending on values like aParallel, etc
		// the rod approximation will give NaN... hence the IF statement below
		int minMonomerCt = 30;
		double minLength;
		if (filAtEnd1 | filAtEnd2) { 
			minLength = Env.stdSegLength.getIntValue()*Env.monomerDiameter.getValue()/2;
		} else {
			minLength = minMonomerCt*Env.monomerDiameter.getValue()/2;
		}
		double asIfLength = length;	// default, use actual length
		if (asIfLength < minLength) { asIfLength = minLength; }
		
		double asIfLengthM = 1.0e-6*asIfLength; // in meters
		double radiusM = Env.filamentRadius.getValue()*1.0e-6;
		
		bTransGam.x = cParallel*asIfLengthM;
		bTransGam.y = cPerp*asIfLengthM;
		bTransGam.z = bTransGam.y;
		bRotGam.x = 4*Math.PI*Env.aeta.getValue()*radiusM*radiusM*asIfLengthM;	// drag for turning about x
		bRotGam.y = cRot*Math.pow(asIfLengthM, 3)/4;
		bRotGam.z = bRotGam.y;
		
		bTransDiff.div(Env.Boltz*Env.tempK, bTransGam);	// Einstein's relation D=kT/gamma
		bRotDiff.div(Env.Boltz*Env.tempK, bRotGam);
		
		updateGraphicsLength = true;
	}
	
	public static void calculateDragTerms () {
		//if drags are for filaments near plane (Howard Chapt 6)
		cParallel = 2*Math.PI*Env.aeta.getValue()/(Math.log(2*Env.hFromPlane.getValue()/(Env.monomerDiameter.getValue()/2))); // per unit length
		cPerp = 2*cParallel;
		cRot = cPerp/3;
	}
	
	public void initialize () {
		// this method assumes the unit x and y vectors have been set (though maybe not orthogonal), or are unchanged
		// determine z-unit vectors, then reset y-unit vector to ensure orthogonality with uVec
		zVec.cross(uVec, yVec);
		yVec.cross(zVec, uVec);
		// find the transformation matrices at this time step
		transMat ();
		// define opposite to uVec direction, used frequently
		uVecR.scale(-1,uVec);
		// length may have changed due to poly/depoly/split
		length=(monomerCt+1)*Env.monomerDiameter.getValue()/2;
		// refind the end points of the rod to make sure they meet length criteria
		end1.add(coord, -length/2, uVec);
		end2.add(coord, length/2, uVec);
		
	}
	
	public void translate (double dist, Pt3D vec) {
		coord.inc(dist,vec);
		end1.inc(dist,vec);
		end2.inc(dist,vec);
	}
	
	public void translateCoord (double dist, Pt3D vec) {
		coord.inc(dist,vec);
	}
	
	public void step () {
		// resolve mechanical interaction and simulate biochemistry over the next time-step
		if (!filAtEnd1 | !filAtEnd2) { checkPlasmidBinding(); }		// any segment with a free end might collide with a plasmid.... if so then bind
		if (!Env.cantileverElasticityTst) { checkPlasmidCollision(); }
			
		addLinkForces();				// if linked to other segments
		
		addTorsionSpringForces();		// bending rigidity proxy
		
		addPlasmidForces();				// calculate elastic forces to keep filament ends and bound plasmids together
		
		moveThing();					// apply all collision and tethering forces to update this Things' coordinates
			
		// split into segments.... only at very start of simulation. 
		if (monomerCt >= 2*Env.stdSegLength.getIntValue()) { 
			splitSegment();
			calculateProperties();	// again if split
			initialize();
		}	
		
		resetCounters();				// reset counters, zero force sums, etc in preparation for the next time step
	}
	
	public void moveThing () {
		// Given the forces/torques at this time point... move with explicit Euler approximation to ODE solution
			                   
		if (Env.cantileverElasticityTst & !filAtEnd2 & Env.simulationTime > 10*Env.deltaT.getValue()) { cantileverSupportTesting1stMode(this); }	// comment out perhaps when running sims
		if (Env.simpleSupportElasticityTst) {
			if (Env.secondMode) {
				if (filArrayPos == Env.forceSeg1 & Env.simulationTime > 10*Env.deltaT.getValue()) { simpleSupportTesting2ndMode(this); }
				if (filArrayPos == Env.forceSeg2 & Env.simulationTime > 10*Env.deltaT.getValue()) { simpleSupportTesting2ndMode(this); }
			} else {
				if (filArrayPos == Env.forceSeg1 & Env.simulationTime > 10*Env.deltaT.getValue()) { simpleSupportTesting1stMode(this); }
			}
		}
		if (Env.filUnderTraction & (Env.simulationTime > 10*Env.deltaT.getValue())) {
			// force entered by user at pN, convert to N
			if (!filAtEnd2) { forceSum.inc(new Pt3D(Env.tracForce.getValue()*1e-12,0,0)); }
			if (!filAtEnd1) { forceSum.inc(new Pt3D(-Env.tracForce.getValue()*1e-12,0,0)); }
		}
		
		// first check that forceSum and torqueSum aren't wacky... exit method if they are
		if (!forceSum.checkPt3D()) { 
			//talkln ("Crazy forceSum in FilSegment"); 
			forceSum.zero();
			if (!Env.filUnderTraction) { Env.filBrownianMotionOff = false;}
		}
		if (!torqueSum.checkPt3D()) { 
			//talkln ("Crazy torqueSum in FilSegment"); 
			torqueSum.zero();
			if (!Env.filUnderTraction) { Env.filBrownianMotionOff = false;}
		}
		
		// Work in coordinates aligned with the rod... transform forces and torques into body-fixed axis....
		bForceSum.XTox(this, forceSum);
		bTorqueSum.XTox(this, torqueSum);

		// add these instantaneous body-fixed forces to the tracked list, then reset them as average value
		/*bForceTrack.registerValue(bForceSum);
		bTorqueTrack.registerValue(bTorqueSum);
		bForceSum.copy(bForceTrack.averagePtVal());
		bTorqueSum.copy(bTorqueTrack.averagePtVal());
		*/
		
		// add brownian force and torque... these are zero except at every chosen time-step
		if (!Env.filBrownianMotionOff) {
			// trans
			double transScale = Env.BTransCoeff.getValue();
			if (!filAtEnd1 & !filAtEnd2) { transScale = 1.0; }	// for unattached segment use rigid rod value
			if (plasmidAtEnd1 | plasmidAtEnd2) { transScale = Env.plasmidBoundTransScale; }
			bForceSum.inc(transScale,randForces);
			
			// rot
			double rotScale = Env.BRotCoeff.getValue();
			if (!filAtEnd1 & !filAtEnd2) { rotScale = 1.0; }	// for unattached segment use rigid rod value
			if ((!filAtEnd1 & !plasmidAtEnd1) | (!filAtEnd2 & !plasmidAtEnd2)) {			// only apply brownian torques to end filaments.. best matches expected angular correlations
				bTorqueSum.inc(rotScale,randTorques);
			}
		}
		// now that the forces and torques are in the body fixed frame, we apply the eoms....
		bVeloc.div(1.0e6, bForceSum, bTransGam);		// in micron/sec
		bAngVeloc.div(bTorqueSum, bRotGam);			// in radians/sec
		
		// ** before progressing .... check that bVeloc and bAngVeloc are not NaN... exit if wacky
		if (!bVeloc.checkPt3D()) { talkln ("** problem with bVeloc for filament " + String.valueOf(this)); return; }
		if (!bAngVeloc.checkPt3D()) { talkln ("** problem with bAngVeloc for filament " + String.valueOf(this)); return; }
		
		// New Positions
		// the body-fixed angular velocities can just be transformed into fixed-frame velocities, and the coord updated
		veloc.xToX(this, bVeloc);
		coord.inc(Env.deltaT.getValue(),veloc);  // just position = velocity*time
		
		//deltaBAng.inc(Env.deltaT.getValue(),bAngVeloc);
		// to apply the body-fixed angular velocities, approximate new unit vector from arc of rotations.. good for small rotations
		// for uVec
		double uVecTransInZ = -bAngVeloc.y * Env.deltaT.getValue();	// arclength out at 1 micron
		double uVecTransInY = bAngVeloc.z * Env.deltaT.getValue();
		uVec.setVals(1,uVecTransInY,uVecTransInZ);	// in body-fixed, not a unit vector yet
		uVec.xToX(this);	// make in fixed-frame, not a unit vector yet
		uVec.unitVec();		// make a unit vector
		
		// for yVec 
		double yVecTransInX = - uVecTransInY;
		double yVecTransInZ = bAngVeloc.x * Env.deltaT.getValue();	// arclength at 1 micron
		yVec.setVals(yVecTransInX, 1, yVecTransInZ);
		yVec.xToX(this);
		yVec.unitVec();
		
		initialize();
		
	}
	
	public void splitSegment () {
		int halfSegCt = Env.stdSegLength.getIntValue();
		monomerCt = monomerCt - halfSegCt;
		length = (monomerCt+1)*Env.monomerDiameter.getValue()/2;
		coord.add(end1,0.5*length,uVec);
		end2.add(end1, length, uVec);
		double nextFilLength = (halfSegCt+1)*Env.monomerDiameter.getValue()/2;
		Pt3D nextFilCoord = Pt3D.Add(end2,0.5*nextFilLength-Env.monomerDiameter.getValue()/2,uVec);
		FilSegment nextFil = new FilSegment (nextFilCoord,this.uVec,halfSegCt,this);
		setEnd2Links(nextFil, true);
	}
	
	public void resetCounters() {
		super.resetCounters();	// call the generic Thing method
		end1AxialF = 0;			// reset axial force at end1 to zero
		end2AxialF = 0; 			// reset axial force at end2 to zero
		lengthChanged = false;	// 
		end1LinkCkd = false;
		end2LinkCkd = false;
		end1TorqCkd = false;
		end2TorqCkd = false;
	}
	
	public double moveCoeff (int end, Pt3D linkUVec) {
		double cosBeta;
		if (end == 2) {
			cosBeta = Pt3D.Dot(uVec, linkUVec);
		} else {
			cosBeta = Pt3D.Dot(uVecR, linkUVec);
		}
		double cosAlpha = Pt3D.OppTrig(cosBeta);
		double lSqrd = 1e-12*length*length;
		double Cx = cosBeta*cosBeta/bTransGam.x;
		double Cperp = cosAlpha*cosAlpha/bTransGam.y;
		double Ctheta = lSqrd*cosAlpha*cosAlpha/(4*bRotGam.y);
		double moveC = Cx + Cperp + Ctheta;
		return moveC;
	}
	
		
	public void addLinkForces () {
		// PAIRS method for enforcing linked endpoint (see Alberts (2008))
		// this filament takes care of both end links, if not already visited, then marks as visited on this and linked fils
		// this method doesn't care if ptAtEnd2 is end2Fil.end1 or end2Fil.end2
	
		if (filAtEnd2 & !end2LinkCkd) {
			linkPt.add(end2,Env.monomerDiameter.getValue()/2,uVecR);		// link point is half a monomer back from end2 tip
			double strainDist = Pt3D.ptDist(linkPt,ptAtEnd2);
			
			linkUVec.unitVec(strainDist,ptAtEnd2,linkPt);
			linkUVecR.scale(-1,linkUVec);
			
			double moveCoeff1 = moveCoeff(2,linkUVec);
			double moveCoeff2;
			if (ptAtEnd2 == end2Fil.end1) {
				moveCoeff2 = end2Fil.moveCoeff(1, linkUVecR);
			} else {
				moveCoeff2 = end2Fil.moveCoeff(2, linkUVecR);
			}
			double forceMag = (Env.CTrans.getValue()*1.0e-6*strainDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
			F.scale(forceMag,linkUVec);
			
			incForceSum(F);
			R.scale(0.5e-6*length*Env.CsubR.getValue(),uVec);
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			end2LinkCkd = true;
			
			Fopp.scale(-1,F);
			end2Fil.incForceSum(Fopp);
			if (ptAtEnd2 == end2Fil.end1) {
				R.scale(0.5e-6*end2Fil.length*Env.CsubR.getValue(),end2Fil.uVecR);
				end2Fil.end1LinkCkd = true;
			} else {
				R.scale(0.5e-6*end2Fil.length*Env.CsubR.getValue(),end2Fil.uVec);
				end2Fil.end2LinkCkd = true;
			}
			RcrossF.cross(R,Fopp);
			end2Fil.incTorqueSum(RcrossF);
			
			// propagate a change in filID.... lower filID always used
			if (filID != end2Fil.filID) {
				if (filID < end2Fil.filID) { end2Fil.filID = filID; } else { filID = end2Fil.filID; }
			}
		}
		
		// take care of link at end1
		if (filAtEnd1 & !end1LinkCkd) {
			linkPt.add(end1,Env.monomerDiameter.getValue()/2,uVec);		// link point is half a monomer back from end2 tip
			double strainDist = Pt3D.ptDist(linkPt,ptAtEnd1);
			linkUVec.unitVec(strainDist,ptAtEnd1,linkPt);
			linkUVecR.scale(-1,linkUVec);
			
			double moveCoeff1 = moveCoeff(1,linkUVec);
			double moveCoeff2;
			if (ptAtEnd1 == end1Fil.end1) {
				moveCoeff2 = end1Fil.moveCoeff(1, linkUVecR);
			} else {
				moveCoeff2 = end1Fil.moveCoeff(2, linkUVecR);
			}
			double forceMag = (Env.CTrans.getValue()*1.0e-6*strainDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
			F.scale(forceMag,linkUVec);
			
			incForceSum(F);
			R.scale(0.5e-6*length*Env.CsubR.getValue(),uVecR);
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			end1LinkCkd = true;
			
			Fopp.scale(-1,F);
			end1Fil.incForceSum(Fopp);
			if (ptAtEnd1 == end1Fil.end1) {
				R.scale(0.5e-6*end1Fil.length*Env.CsubR.getValue(),end1Fil.uVecR);
				end1Fil.end1LinkCkd = true;
			} else {
				R.scale(0.5e-6*end1Fil.length*Env.CsubR.getValue(),end1Fil.uVec);
				end1Fil.end2LinkCkd = true;
			}

			RcrossF.cross(R,Fopp);
			end1Fil.incTorqueSum(RcrossF);
			
			// propagate a change in filID.... lower filID always used
			if (filID != end1Fil.filID) {
				if (filID < end1Fil.filID) { end1Fil.filID = filID; } else { filID = end1Fil.filID; }
			}
		}
	}

	
	public void addTorsionSpringForces () {
		// rotational spring which works to straighten out the filament
		// this filament takes care of spring at both ends, if not visited, then marks as visited for involved fils
		// this method doesn't care if ptAtEnd2 is end2Fil.end1 or end2Fil.end2
		
		if (filAtEnd2 & !end2TorqCkd) {
			end2TorqCkd = true;
			double dotVecs;
			if (ptAtEnd2 == end2Fil.end1) {
				end2Fil.end1TorqCkd = true;
				torsionVec.cross(uVec,end2Fil.uVec);
				torsionVec.unitVec();
				dotVecs = Pt3D.Dot(uVec,end2Fil.uVec);
			} else {
				end2Fil.end2TorqCkd = true;
				torsionVec.cross(uVec,end2Fil.uVecR);
				torsionVec.unitVec();
				dotVecs = Pt3D.Dot(uVec,end2Fil.uVecR);
			}
			
			if (dotVecs > 1.0) { dotVecs = 1.0; }
			double angTween = Math.acos(dotVecs)*180/Math.PI;
			double torsionMag;
			if (Env.filTorqSpring.isActive()) {
				torsionMag = Env.CsubTheta.getValue()*Env.filTorqSpring.getValue()*angTween;
			} else { 
				//torsionMag = Env.fracMoveTorq.getValue()*(Math.PI/180)*end2SegAng.averageVal()/((1/bRotGam.y + 1/end2Fil.bRotGam.y)*Env.deltaT.getValue());
				torsionMag = Env.CsubTheta.getValue()*(Math.PI/180)*angTween/((1/bRotGam.y + 1/end2Fil.bRotGam.y)*Env.deltaT.getValue());
			}
		
			if (torsionVec.checkPt3D()) {
				torsionVec.scale(torsionMag);
				incTorqueSum(torsionVec);
			
				torsionVec.scale(-1);
				end2Fil.incTorqueSum(torsionVec);
			} else {
				talkln ("Crazy torque result in FilSegment.addTorsionSpringForce() part 2");
			}
		}
		
		
		if (filAtEnd1 & !end1TorqCkd) {
			end1TorqCkd = true;
			double dotVecs;
			if (ptAtEnd1 == end1Fil.end1) {
				end1Fil.end1TorqCkd = true;
				torsionVec.cross(uVecR,end1Fil.uVec);
				torsionVec.unitVec();
				dotVecs = Pt3D.Dot(uVecR,end1Fil.uVec);
			} else {
				end1Fil.end2TorqCkd = true;
				torsionVec.cross(uVecR,end1Fil.uVecR);
				torsionVec.unitVec();
				dotVecs = Pt3D.Dot(uVecR,end1Fil.uVecR);
			}
			
			if (dotVecs > 1.0) { dotVecs = 1.0; }
			double angTween = Math.acos(dotVecs)*180/Math.PI;
			double torsionMag;
			if (Env.filTorqSpring.isActive()) {
				torsionMag = Env.CsubTheta.getValue()*Env.filTorqSpring.getValue()*angTween;
			} else { 
				torsionMag = Env.CsubTheta.getValue()*(Math.PI/180)*angTween/((1/bRotGam.y + 1/end1Fil.bRotGam.y)*Env.deltaT.getValue());
			}
			if (torsionVec.checkPt3D()) {
				torsionVec.scale(torsionMag);
				incTorqueSum(torsionVec);
				
				torsionVec.scale(-1);
				end1Fil.incTorqueSum(torsionVec);
			} else {
				talkln ("Crazy torque result in FilSegment.addTorsionSpringForce() part 1");
			}
		}
	}
	
	public boolean ptInPlasmidBoundingBox (Pt3D pt, PSphere plas) {
		// quicker checks to see if this end could be colliding with this plasmid... bounding box
		double cushion = Env.monomerDiameter.getValue();
		double plasRad = plas.getRadius()+cushion;
		// x coord
		if (pt.x < plas.coord.x - plasRad) { return false; }
		if (pt.x > plas.coord.x + plasRad) { return false; }
		// y coord
		if (pt.y < plas.coord.y - plasRad) { return false; }
		if (pt.y > plas.coord.y + plasRad) { return false; }
		// z coord
		if (pt.z < plas.coord.z - plasRad) { return false; }
		if (pt.z > plas.coord.z + plasRad) { return false; }
		
		return true;
	}
	
	public void checkPlasmidCollision() {
		// if colliding with plasmid at end1 then push away
		for (int i=0;i<PSphere.sphereCt;i++){
			PSphere curPlasmid = PSphere.theSpheres[i];
			if (ptInPlasmidBoundingBox(end1,curPlasmid)) {
				double distToEnd1 = Pt3D.ptDist(curPlasmid.coord,end1);
				double impDist = curPlasmid.getRadius() - distToEnd1;
				//if (impDist > -halfmono) { end1TipC = 0; } 	// steric hindrance to polymerization set
				if (impDist > 0) {
					linkUVec.unitVec(distToEnd1,end1,curPlasmid.coord);
					linkUVecR.scale(-1,linkUVec);
					// define cosines of angles between filament uVecs and line between endpoints
					double cosAngTween1 = Pt3D.CrossMag(uVecR,linkUVec);  // use magnitude of cross product (which is Sin(theta)) 'cause we want Cos(90-theta)=Sin(theta)
					double moveCoeff1 = 1/bTransGam.x + Math.pow(1e-6*length*cosAngTween1/2,2)/bRotGam.y;
					double moveCoeff2 = 1/curPlasmid.bTransGam.x;
					double forceMag = (Env.CTrans.getValue()*1.0e-6*impDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
					
					F.scale(forceMag,linkUVec);
					incForceSum(F);
					R.scale(1e-6*length/2,uVecR);
					RcrossF.cross(R,F);
					incTorqueSum(RcrossF);
					
					Fopp.scale(-1,F);
					curPlasmid.incForceSum(Fopp);
					// note: no torque on plasmid.... force through CM
				}
			}
		}

		// if colliding with plasmid at end2 then push away
		for (int i=0;i<PSphere.sphereCt;i++){
			PSphere curPlasmid = PSphere.theSpheres[i];
			if (ptInPlasmidBoundingBox(end2,curPlasmid)) {
				double distToEnd2 = Pt3D.ptDist(curPlasmid.coord,end2);
				double impDist = curPlasmid.getRadius() - distToEnd2;
				//if (impDist > -halfmono) { end2TipC = 0; } 	// steric hindrance to polymerization set
				if (impDist > 0) {
					linkUVec.unitVec(distToEnd2,end2,curPlasmid.coord);
					linkUVecR.scale(-1,linkUVec);
					// define cosines of angles between filament uVecs and line between endpoints
					double cosAngTween1 = Pt3D.CrossMag(uVec,linkUVec);  // use magnitude of cross product (which is Sin(theta)) 'cause we want Cos(90-theta)=Sin(theta)
					double moveCoeff1 = 1/bTransGam.x + Math.pow(1e-6*length*cosAngTween1/2,2)/bRotGam.y;
					double moveCoeff2 = 1/curPlasmid.bTransGam.x;
					double forceMag = (Env.CTrans.getValue()*1.0e-6*impDist)/(Env.deltaT.getValue()*(moveCoeff1 + moveCoeff2));
					
					F.scale(forceMag,linkUVec);
					incForceSum(F);
					R.scale(1e-6*length/2,uVec);
					RcrossF.cross(R,F);
					incTorqueSum(RcrossF);
					
					Fopp.scale(-1,F);
					curPlasmid.incForceSum(Fopp);
					// note: no torque on plasmid.... force through CM
				}
			}
		}
	}
	
	public void checkPlasmidBinding() {
		if (Env.end1CanBindPlasmid.isActive()) { checkPlasmidBindingEnd1(); }
		if (Env.end2CanBindPlasmid.isActive()) { checkPlasmidBindingEnd2(); }
	}
	
	public void checkPlasmidBindingEnd1() {
		if ((!plasmidAtEnd1) & (!end1Catastrophy) & (!filAtEnd1) & (end1DetachCounter == 0)) {
			for (int i=0;i<PSphere.sphereCt;i++){
			PSphere curPlasmid = PSphere.theSpheres[i];
				double distplasmid = Pt3D.ptDist(curPlasmid.coord,end1);
				if ((distplasmid<curPlasmid.getRadius()) & (curPlasmid != end2Plasmid)) {
					plasmidAtEnd1= true;
					end1Plasmid=curPlasmid;
					linkUVec.sub(curPlasmid.getRadius()/distplasmid,end1,curPlasmid.coord);	// vector to edge of plasmid in direction from coord to end1
					end1PAttachPt.XTox(curPlasmid,linkUVec);
					curPlasmid.filamentOn();
					//gotPlasmidAppearance();
					setGlobalEnd1Plasmid(true);
				}
			}
		}

	}
	
	public void checkPlasmidBindingEnd2() {
		if ((!plasmidAtEnd2) & (!end2Catastrophy) & (!filAtEnd2) & (end2DetachCounter ==0)) {
			for (int i=0;i<PSphere.sphereCt;i++){
				PSphere curPlasmid = PSphere.theSpheres[i];
				double distplasmid = Pt3D.ptDist(curPlasmid.coord,end2);
				if ((distplasmid<curPlasmid.getRadius()) & (curPlasmid != end1Plasmid)) {
					plasmidAtEnd2= true;
					end2Plasmid=curPlasmid;
					linkUVec.sub(curPlasmid.getRadius()/distplasmid,end2,curPlasmid.coord);	// vector to edge of plasmid in direction from coord to end2
					end2PAttachPt.XTox(curPlasmid,linkUVec);
					curPlasmid.filamentOn();
					//gotPlasmidAppearance();
					setGlobalEnd2Plasmid(true);
				}
			}
		}
	}
	
	public void addPlasmidForces() {
		if (plasmidAtEnd1) {
			end1Plasmid.registerFilament(this);
			
			end1PAttachPtInX.xToXPlusxOrigin(end1Plasmid,end1PAttachPt);
			double strainDist = Pt3D.ptDist(end1PAttachPtInX,end1);
			double forceMag = Env.CTrans.getValue()*1.0e-6*strainDist/((1/bTransGam.x + 1/end1Plasmid.bTransGam.x)*Env.deltaT.getValue());
			//double forceMag=Env.kParM_PlasmidSpring*(Pt3D.ptDist(end1Plasmid.coord,end1));
			toPlasmidUVec.unitVec(end1PAttachPtInX,end1);
			F.scale(forceMag,toPlasmidUVec);
			incForceSum(F);
			R.scale(1e-6*(0.5*length),uVecR);
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			
			Fopp.scale(-1,F);
			end1Plasmid.incForceSum(Fopp);
			R.sub(1e-6,end1PAttachPtInX,end1Plasmid.coord);
			RcrossF.cross(R,Fopp);
			end1Plasmid.incTorqueSum(RcrossF);
			
			// torsional spring between plasmid and segment
			toPlasmidUVec.unitVec(end1,end1Plasmid.coord);	// the unit vector orthogonal to the plasmid at the attachment point
			torsionVec.cross(uVec,toPlasmidUVec);
			torsionVec.unitVec();
			double angTween = Math.acos(Pt3D.Dot(uVec,toPlasmidUVec));
			double torsionMag;
			if (Env.plasmidTorqSpring.isActive()) {
				double kTorque = Env.plasmidTorqSpring.getValue();
				if (Env.simpleSupportElasticityTst) { kTorque = 0; }
				torsionMag = kTorque*angTween;
			} else {
				torsionMag = Env.CsubTheta.getValue()*angTween*bRotGam.y/Env.deltaT.getValue();
			}
			torsionVec.scale(torsionMag);
			incTorqueSum(torsionVec);	
			torsionVec.scale(-1);
			end1Plasmid.incTorqueSum(torsionVec);
			
			
			if (releaseAllPlasmids) {
				//talkln ("removing plasmid tether at end1");
				plasmidAtEnd1= false;
				end1Plasmid.filamentOff();
				end1Plasmid=null;
				end1PAttachPt.zero();
				end1DetachCounter = 10000;
				setGlobalEnd1Plasmid(false);
			}
		
		}
		if (plasmidAtEnd2) {
			end2Plasmid.registerFilament(this);
			
			end2PAttachPtInX.xToXPlusxOrigin(end2Plasmid,end2PAttachPt);
			double strainDist = Pt3D.ptDist(end2PAttachPtInX,end2);
			double forceMag = Env.CTrans.getValue()*1.0e-6*strainDist/((1/bTransGam.x + 1/end2Plasmid.bTransGam.x)*Env.deltaT.getValue());
			//double forceMag=Env.kParM_PlasmidSpring*(Pt3D.ptDist(end2Plasmid.coord,end2));
			toPlasmidUVec.unitVec(end2PAttachPtInX,end2);
			F.scale(forceMag,toPlasmidUVec);
			incForceSum(F);
			R.scale(1e-6*(0.5*length),uVec);
			RcrossF.cross(R,F);
			incTorqueSum(RcrossF);
			
			Fopp.scale(-1,F);
			end2Plasmid.incForceSum(Fopp);
			R.sub(1e-6,end2PAttachPtInX,end2Plasmid.coord);
			RcrossF.cross(R,Fopp);
			end2Plasmid.incTorqueSum(RcrossF);
			
			//	torsional spring between plasmid and segment
			toPlasmidUVec.unitVec(end2,end2Plasmid.coord);	// the unit vector orthogonal to the plasmid at the attachment point
			torsionVec.cross(uVecR,toPlasmidUVec);
			torsionVec.unitVec();
			double angTween = Math.acos(Pt3D.Dot(uVecR,toPlasmidUVec));
			double torsionMag;
			if (Env.plasmidTorqSpring.isActive()) {
				double kTorque = Env.plasmidTorqSpring.getValue();
				if (Env.simpleSupportElasticityTst) { kTorque = 0; }
				torsionMag = kTorque*angTween;
			} else {
				torsionMag = -Env.CsubTheta.getValue()*angTween*bRotGam.y/Env.deltaT.getValue();
			}
			torsionVec.scale(torsionMag);
			incTorqueSum(torsionVec);	
			torsionVec.scale(-1);
			end2Plasmid.incTorqueSum(torsionVec);
			
			if (releaseAllPlasmids) {
				//talkln ("removing plasmid tether at end2");
				plasmidAtEnd2= false;
				end2Plasmid.filamentOff();
				end2Plasmid=null;
				end2PAttachPt.zero();
				end2DetachCounter = 10000;
				setGlobalEnd2Plasmid(false);
			}
			
		}
	}
	
	public void setGlobalEnd1Plasmid (boolean onState) {
		if (true) { return; }
		FilSegment curSeg = this;
		while (curSeg.filAtEnd2) {
			curSeg.globalPlasAtEnd1 = onState;
			curSeg = curSeg.end2Fil;
		}
		curSeg.globalPlasAtEnd1 = onState; // get last one
	}
	
	public void setGlobalEnd2Plasmid (boolean onState) {
		if (true) { return; }
		FilSegment curSeg = this;
		while (curSeg.filAtEnd1) {
			curSeg.globalPlasAtEnd2 = onState;
			curSeg = curSeg.end1Fil;
		}
		curSeg.globalPlasAtEnd2 = onState; // get end filSegment too
	}
	
	
	
	public static void linkEnd1Plasmid (FilSegment fil, PSphere plas) {
		fil.plasmidAtEnd1 = true;
		fil.end1Plasmid = plas;
		plas.filamentOn();
	}
	
	public static void linkEnd2Plasmid (FilSegment fil, PSphere plas) {
		fil.plasmidAtEnd2 = true;
		fil.end2Plasmid = plas;
		plas.filamentOn();
	}
	
	public static void link (FilSegment at1, FilSegment at2) {
		at1.setEnd2Links(at2,true);
		at2.setEnd1Links(at1, true);
	}
	
	public void setEnd1Links (FilSegment at1, boolean normOrientation) {
		filAtEnd1 = true;
		end1Fil = at1;
		if (normOrientation) { ptAtEnd1 = at1.end2; } else { ptAtEnd1 = at1.end1; }
	}
	
	public void setEnd2Links (FilSegment at2, boolean normOrientation) {
		filAtEnd2 = true;
		end2Fil = at2;
		if (normOrientation) { ptAtEnd2 = at2.end1; } else { ptAtEnd2 = at2.end2; }
	}
	
	public static void transferEnd1Plasmid (FilSegment givesP, FilSegment getsP) {
		getsP.plasmidAtEnd1 = true;
		getsP.end1Plasmid = givesP.end1Plasmid;
		getsP.end1PAttachPt.copy(givesP.end1PAttachPt);
		givesP.plasmidAtEnd1=false;
		givesP.end1Plasmid=null;
	}
	
	public static void transferEnd2Plasmid (FilSegment givesP, FilSegment getsP) {
		getsP.plasmidAtEnd2 = true;
		getsP.end2Plasmid = givesP.end2Plasmid;
		getsP.end2PAttachPt.copy(givesP.end2PAttachPt);
		givesP.plasmidAtEnd2=false;
		givesP.end2Plasmid=null;
	}
		
	public static void cleanup (FilSegment cleanF, boolean swapOut) {
			if (!cleanF.filAtEnd1 & !cleanF.filAtEnd2) { filCt--; }		// only decrement filament counter if unattached to other FilSegments
		
			if (cleanF.plasmidAtEnd1) {
				if (cleanF.filAtEnd2) {
					transferEnd1Plasmid(cleanF,cleanF.end2Fil);
				} else {
					cleanF.end1Plasmid.filamentOff(); 
				}
			}
			if (cleanF.plasmidAtEnd2) { 
				if (cleanF.filAtEnd1) {
					transferEnd2Plasmid(cleanF,cleanF.end1Fil);
				} else {
					cleanF.end2Plasmid.filamentOff(); 
				}
			}
		
			if (swapOut) { removeFilSegment(cleanF); }
	}
	
	public static void removeAll () {
		for (int i=0;i<filSegmentCt;i++) {
			cleanup(theFilSegments[i],false);
			theFilSegments[i].G.detach();
			theFilSegments[i].end1BG.detach();
			theFilSegments[i].end2BG.detach();
			theFilSegments[i].coordBG.detach();
			theFilSegments[i].coordSysG.detach();
		}
		for (int i=0;i<filSegmentCt;i++) {
			theFilSegments[i].removeMe = true;
			theFilSegments[i] = null;
		}
		filSegmentCt = 0;	// reset array counter
		filCt = 0;	// reset unique id counter
	}
	
	public static void cantileverSupportTesting1stMode (FilSegment curFil) {
		Env.plasmidBrownianMotionOff = true;
		Env.filBrownianMotionOff = true;
		Env.currentDeflection = curFil.end2.y;
		if (!Env.relaxCantilever) { 
			Env.reportedTimeConstant = false;
			curFil.forceSum.add(Env.staticForce); 
		} else {
			if (!Env.reportedTimeConstant & Env.currentDeflection > Env.relaxationDeflection) {
				Env.reportedTimeConstant = true;
				talkln ("tau cantilever 1st mode = " + Env.simulationTime + " ; expected tau = " + expectedTauCant1stMode());
			}
			if (Env.relaxWriteCounter >= Env.relaxWriteInterval) {
				//FileOps_.writeToRelaxFile();
				Env.relaxWriteCounter = 0;
			} else {
				Env.relaxWriteCounter++;
			}
		}
	}
	
	public static void simpleSupportTesting1stMode (FilSegment curFil) {
		Env.plasmidBrownianMotionOff = true;
		Env.filBrownianMotionOff = true;
		if (Env.relaxSimpleSupport | Env.simulationTime > 100*Env.deltaT.getValue()) {
			FilSegment end1Seg = FilSegment.theFilSegments[0];
			FilSegment end2Seg = FilSegment.theFilSegments[Env.curMultipleOfSeg-1];
			double onLineY = (end1Seg.coord.y + end2Seg.coord.y)/2;
			Env.currentDeflection = curFil.coord.y-onLineY;
		}
		
		// automatic tuning.... turned off for now
		/*if (Env.simulationTime > 100*Env.deltaT.getValue()) {
			Env.deltaTrackCounter++;
			if (Env.deltaTrackCounter > Env.deltaTrackInt) {
				double deltaD = Math.abs(Env.currentDeflection-Env.lastDeflection);
				double deltaE = Math.abs(Env.currentDeflection-Env.expectedDeflection);
				
				if (deltaD < Env.deltaTolerance) {
					if (deltaE < Env.deltaTolerance) {	// we're done with this tuning
						talkln ("Done with this tuning");
						
					} else {	// stagnant
						if (Env.currentDeflection > Env.expectedDeflection) {
							Env.CsubTheta.decValue(Env.curDeltaCtheta);
							if (Env.CsubTheta.getValue() < 0) { Env.CsubTheta.setValue(0); }
						} else {
							Env.CsubTheta.incValue(Env.curDeltaCtheta);
							if (Env.CsubTheta.getValue() > 0.6) { Env.CsubTheta.setValue(0.6); }
						}
					}
				}
				
				// check the direction of change... alter tuning
				Env.goingPlus = (Env.currentDeflection - Env.lastDeflection) > 0;
				Env.shouldGoPlus = (Env.expectedDeflection - Env.currentDeflection) > 0;
				if (Env.goingPlus & !Env.shouldGoPlus) {
					if (!Env.lastSwitchPlus) { Env.curDeltaCtheta = Env.curDeltaCtheta/2; }
					Env.lastSwitchPlus = true;
					Env.CsubTheta.decValue(Env.curDeltaCtheta);
					if (Env.CsubTheta.getValue() < 0) { Env.CsubTheta.setValue(0); }
					talkln ("switching: goingPlus = " + Env.goingPlus + " ; shouldGoPlus = " + Env.shouldGoPlus);
				}
				
				if (!Env.goingPlus & Env.shouldGoPlus) {
					if (Env.lastSwitchPlus) { Env.curDeltaCtheta = Env.curDeltaCtheta/2; }
					Env.lastSwitchPlus = false;
					Env.CsubTheta.incValue(Env.curDeltaCtheta);
					if (Env.CsubTheta.getValue() > 0.6) { Env.CsubTheta.setValue(0.6); }
					talkln ("switching: goingPlus = " + Env.goingPlus + " ; shouldGoPlus = " + Env.shouldGoPlus);
				}
				
				Env.deltaTrackCounter = 0;
				Env.lastDeflection = Env.currentDeflection;
			}
		}*/
		
		if (!Env.relaxSimpleSupport) { 
			Env.reportedTimeConstant = false;
			curFil.forceSum.inc(Env.staticForce);
		} else {
			if (!Env.reportedTimeConstant & Env.currentDeflection > Env.relaxationDeflection) {
				Env.reportedTimeConstant = true;
				talkln ("tau SS 1st mode = " + Env.simulationTime + " ; expected tau = " + expectedTauSS1stMode());
			}
			if (Env.relaxWriteCounter >= Env.relaxWriteInterval) {
				//FileOps.writeToRelaxFile();
				Env.relaxWriteCounter = 0;
			} else {
				Env.relaxWriteCounter++;
			}
		}
	}
	
	public static void simpleSupportTesting2ndMode (FilSegment curFil) {
		Env.plasmidBrownianMotionOff = true;
		Env.filBrownianMotionOff = true;
		if (Env.relaxSimpleSupport | Env.simulationTime > 100*Env.deltaT.getValue()) {
			FilSegment end1Seg = FilSegment.theFilSegments[0];
			FilSegment end2Seg = FilSegment.theFilSegments[Env.curMultipleOfSeg-1];
			FilSegment forceSeg1Seg = FilSegment.theFilSegments[Env.forceSeg1];
			double endToEndSlope = (end2Seg.end2.y - end1Seg.end1.y)/(end2Seg.end2.x-end1Seg.end1.x);
			double onLineY = end1Seg.end1.y + (forceSeg1Seg.coord.x - end1Seg.end1.x)*endToEndSlope;
			Env.currentDeflection = forceSeg1Seg.coord.y - onLineY;
		}
		if (!Env.relaxSimpleSupport) { 
			Env.reportedTimeConstant = false;
			if (curFil.filArrayPos == Env.forceSeg1) { curFil.forceSum.inc(3,Env.staticForce); } 
			if (curFil.filArrayPos == Env.forceSeg2) { curFil.forceSum.inc(3,Env.staticForceR); }
		} else {
			if (!Env.reportedTimeConstant & Env.currentDeflection > Env.relaxationDeflection) {
				Env.reportedTimeConstant = true;
				talkln ("tau SS 2nd mode = " + Env.simulationTime + " ; expected tau = " + expectedTauSS2ndMode());
			}
			if (Env.relaxWriteCounter >= Env.relaxWriteInterval) {
				//FileOps_.writeToRelaxFile();
				Env.relaxWriteCounter = 0;
			} else {
				Env.relaxWriteCounter++;
			}
		}
		
	}
	
	public static double expectedTauSS1stMode () {
		double L = Env.filamentLength.getValue()*1e-6;	// (m) length of filament
		double r = Env.filamentRadius.getValue()*1e-6; 	// (m) x-sectional radius of filament
		double aeta = Env.aeta.getValue();	// (Pa-s) viscosity
		double h = Env.hFromPlane.getValue()*1e-6;	// (m) distance of filament off of plane
		double cPerp = 4*Math.PI*aeta/(Math.log(2*h/r));	// transverse translational drag per unit length
		double kB = Env.Boltz;
		double T = Env.tempK;
		double Lp = Env.persistenceLength.getValue()*1e-6;
		double EI = kB*T*Lp;	// EI = kTLp from worm-like chain theory
		double tau = (cPerp/EI)*Math.pow(L/(1.5*Math.PI),4);
		return tau;
	}
	
	public static double expectedTauSS2ndMode () {
		double L = Env.filamentLength.getValue()*1e-6;	// (m) length of filament
		double r = Env.filamentRadius.getValue()*1e-6; 	// (m) x-sectional radius of filament
		double aeta = Env.aeta.getValue();	// (Pa-s) viscosity
		double h = Env.hFromPlane.getValue()*1e-6;	// (m) distance of filament off of plane
		double cPerp = 4*Math.PI*aeta/(Math.log(2*h/r));	// transverse translational drag per unit length
		double kB = Env.Boltz;
		double T = Env.tempK;
		double Lp = Env.persistenceLength.getValue()*1e-6;
		double EI = kB*T*Lp;	// EI = kTLp from worm-like chain theory
		double tau = (cPerp/EI)*Math.pow(L/(2.5*Math.PI),4);
		return tau;
	}
	
	public static double expectedTauCant1stMode () {
		double L = Env.filamentLength.getValue()*1e-6;	// (m) length of filament
		double r = Env.filamentRadius.getValue()*1e-6; 	// (m) x-sectional radius of filament
		double aeta = Env.aeta.getValue();	// (Pa-s) viscosity
		double h = Env.hFromPlane.getValue()*1e-6;	// (m) distance of filament off of plane
		double cPerp = 4*Math.PI*aeta/(Math.log(2*h/r));	// transverse translational drag per unit length
		double kB = Env.Boltz;
		double T = Env.tempK;
		double Lp = Env.persistenceLength.getValue()*1e-6;
		double EI = kB*T*Lp;	// EI = kTLp from worm-like chain theory
		double tau = (cPerp*Math.pow(L,4))/(EI*Math.pow(1.875,4));
		return tau;
	}
	

	
	public static void simpleSupportRelaxActions () { 
		Env.simulationTime = 0;
		releaseAllPlasmids = true;
	}
	
	public static void cantileveredRelaxActions () {
		Env.simulationTime = 0;
	}
			
	
	public static void persistenceTrack () {
		if (FilSegment.filSegmentCt > 0) {
			// end1 segment
			FilSegment end1Seg = FilSegment.theFilSegments[0];
			
			// cycle through filsegments in link order to consistently enter their angle info
			FilSegment curFil = end1Seg;
			double dotUVecs;
			for (int i=0;i<Env.persistenceSegNumber;i++) {
				dotUVecs = Pt3D.Dot(end1Seg.uVec, curFil.uVec);
				Env.persistenceTrack[i].registerValue(dotUVecs);
				if (curFil.end2Fil != null) { curFil = curFil.end2Fil; }
			}
			
			if (Env.logFiles | Env.justLogLp) {
				FileOps_.writeToPersistenceLengthFile();
			} else {
				// write out the current average values
				/*talk ("simTime = " + Env.simulationTime + ";");
				for (int i=0;i<Env.persistenceSegNumber;i++) {
					talk (Env.persistenceTrack[i].runningAverageVal() + ";");
				}
				talkln("");
				*/
			}
		}
		
	}
	
	public static void setFilamentID (FilSegment newFilSeg, int filID) {
		if (filID == -1) {
			newFilSeg.filID = filIDCt;
			filCt++;
			filIDCt++;
		} else {
			newFilSeg.filID = filID;
		}
	}
	
	public static void addFilSegment (FilSegment newFilament) {
		theFilSegments[filSegmentCt] = newFilament;
		newFilament.filArrayPos = filSegmentCt;
		filSegmentCt ++;
	}
	public static void removeFilSegment ( FilSegment byeFilament) {
		int swapId = byeFilament.filArrayPos;
		theFilSegments[swapId] = theFilSegments[filSegmentCt-1];
		theFilSegments[swapId].filArrayPos = swapId;
		filSegmentCt --;
		byeFilament.removeMe = true;
	
	}
	
	private void makeNewCyl () {
		int cylXTess = Env.lowCylXTessalation;
		int cylYTess = Env.lowCylYTessalation;
		int transMode = TransparencyAttributes.NONE;
		switch (Env.graphicsQuality) {
		case Env.LOW_QUALITY:
			cylXTess = Env.lowCylXTessalation;
			cylYTess = Env.lowCylYTessalation;
			//transMode = TransparencyAttributes.NONE;
			break;
		case Env.MID_QUALITY:
			cylXTess = Env.midCylXTessalation;
			cylYTess = Env.midCylYTessalation;
			//transMode = TransparencyAttributes.FASTEST;
			break;
		case Env.HIGH_QUALITY:
			cylXTess = Env.highCylXTessalation;
			cylYTess = Env.highCylYTessalation;
			//transMode = TransparencyAttributes.NICEST;
			break;
		}
		//Appearance 
		Material cylM = new Material(ambientC,emissiveC,diffuseC,specularC,shiny);
		Appearance aCyl = new Appearance();
		TransparencyAttributes tA = new TransparencyAttributes ();
		tA.setTransparency(1.0f);
		tA.setTransparencyMode(transMode);
		aCyl.setTransparencyAttributes(tA);
		aCyl.setMaterial(cylM);
		
		//cylindrical body
		cylBG = new BranchGroup();
		cylBG.setCapability(BranchGroup.ALLOW_DETACH);
		myCyl = new Cylinder((float)(Env.thickenFactor.getValue()*Env.filamentRadius.getValue()),(float)length,Primitive.GENERATE_NORMALS,cylXTess,cylYTess,aCyl);
		cylRot = new Transform3D();
		cylRot.rotZ(Math.PI/2);
		cylT3D = new Transform3D();
		cylT3D.setRotation(mxToX);
		cylT3D.mul(cylRot);
		Pt3D cylCen = Pt3D.Add(end1,length/2,uVec);
		cylT3D.setTranslation(new Vector3d(cylCen.x,cylCen.y,cylCen.z));
		cylTG = new TransformGroup(cylT3D);
		cylTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		cylTG.addChild(myCyl);
		cylBG.addChild(cylTG);
		G.addChild(cylBG);
	}

	public void makeGraphics () {
		if (!graphicsInitialized) {
			catasphApp.setTransparencyAttributes(tA);
			catasphApp.setMaterial(catasphMat);
			//noPlasmidApp.setTransparencyAttributes(tA);
			noPlasmidApp.setMaterial(noPlasmidMat);
			gotPlasmidApp.setTransparencyAttributes(tA);
			gotPlasmidApp.setMaterial(gotPlasmidMat);
			
			xLineApp.setColoringAttributes(xLineCA);
			yLineApp.setColoringAttributes(yLineCA);
			zLineApp.setColoringAttributes(zLineCA);
			graphicsInitialized = true;
		}
		
		setGraphicsCapabilities();
		
		// coordSphere
		coordS = new Sphere ((float)(2*Env.monomerDiameter.getValue()/2),Sphere.GENERATE_NORMALS, Env.parMEndSphereTessalation, noPlasmidApp);
		coordS.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		coordT3D = new Transform3D();
		coordTG = new TransformGroup(coordT3D);
		coordTG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		coordTG.addChild(coordS);
		coordBG = new BranchGroup();
		coordBG.setCapability(BranchGroup.ALLOW_DETACH);
		coordBG.addChild(coordTG);
		
		// end1Sphere
		end1S = new Sphere ((float)(2*Env.monomerDiameter.getValue()/2),Sphere.GENERATE_NORMALS, Env.parMEndSphereTessalation, noPlasmidApp);
		end1S.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		end1T3D = new Transform3D();
		end1TG = new TransformGroup(end1T3D);
		end1TG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		end1TG.addChild(end1S);
		end1BG = new BranchGroup();
		end1BG.setCapability(BranchGroup.ALLOW_DETACH);
		end1BG.addChild(end1TG);
		
		// end2Sphere
		end2S = new Sphere ((float)(2*Env.monomerDiameter.getValue()/2),Sphere.GENERATE_NORMALS, Env.parMEndSphereTessalation, noPlasmidApp);
		end2S.getShape().setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		end2T3D = new Transform3D();
		end2TG = new TransformGroup(end2T3D);
		end2TG.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
		end2TG.addChild(end2S);
		end2BG = new BranchGroup();
		end2BG.setCapability(BranchGroup.ALLOW_DETACH);
		end2BG.addChild(end2TG);
		
		// filament cylinder
		makeNewCyl();
		
		// coordinate system
		// x-axis
		xLine = new LineArray(2,LineArray.COORDINATES);
		xLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		xLineEndPt.add(coord,0.5*length+coordLineLength,uVec);
		xLine.setCoordinate(0,coord);
		xLine.setCoordinate(1,xLineEndPt);
		xLineShape = new Shape3D();
		xLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		xLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		xLineShape.setGeometry(xLine);
		xLineShape.setAppearance(xLineApp);
		// y-axis
		yLine = new LineArray(2,LineArray.COORDINATES);
		yLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		yLineEndPt.add(coord,coordLineLength,yVec);
		yLine.setCoordinate(0,coord);
		yLine.setCoordinate(1,yLineEndPt);
		yLineShape = new Shape3D();
		yLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		yLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		yLineShape.setGeometry(yLine);
		yLineShape.setAppearance(yLineApp);
		// z-axis
		zLine = new LineArray(2,LineArray.COORDINATES);
		zLine.setCapability(LineArray.ALLOW_COORDINATE_WRITE);
		zLineEndPt.add(coord,coordLineLength,zVec);
		zLine.setCoordinate(0,coord);
		zLine.setCoordinate(1,zLineEndPt);
		zLineShape = new Shape3D();
		zLineShape.setCapability(Shape3D.ALLOW_APPEARANCE_WRITE);
		zLineShape.setCapability(Shape3D.ALLOW_GEOMETRY_WRITE);
		zLineShape.setGeometry(zLine);
		zLineShape.setAppearance(zLineApp);
		coordSysG = new BranchGroup();
		coordSysG.setCapability(BranchGroup.ALLOW_DETACH);
		coordSysG.addChild(xLineShape);
		coordSysG.addChild(yLineShape);
		coordSysG.addChild(zLineShape);
		
		if (!Env.helixSpheres) {
			G.addChild(end1BG);
			G.addChild(end2BG);
		}
		
		if (Env.filCoordSysOn) {
			G.addChild(coordBG);
			G.addChild(coordSysG);
			coordSysOn = true;
		}
		
		graphicsMade = true;
	}
	
	public void updateGraphics () {
		if (!graphicsMade) { makeGraphics(); }
		
		if (!plasmidAtEnd1) { end1BG.detach(); }
		if (!plasmidAtEnd2) { end2BG.detach(); }
		if (plasmidAtEnd1 & !end1BGOn) { G.addChild(end1BG);  end1BGOn = true; }
		if (plasmidAtEnd2 & !end2BGOn) { G.addChild(end2BG);  end2BGOn = true; }
		
		boolean thisIsForceSeg = false;
		if (Env.simpleSupportElasticityTst) {
			if (filArrayPos == Env.forceSeg1) { thisIsForceSeg = true; }
			if (filArrayPos == Env.forceSeg2 & Env.secondMode) { thisIsForceSeg = true; }
		}
		if (Env.filCoordSysOn | thisIsForceSeg) { 
			addCoordSysGraphics(); 
		} else { 
			removeCoordSysGraphics();
		}
		
		end1MonCenter.add(end1,Env.monomerDiameter.getValue()/2,uVec);
		end1MonCenter.copyToVector3d(end1Vec3d);
		end1T3D.setTranslation(end1Vec3d);
		end1TG.setTransform(end1T3D);
		
		end2MonCenter.add(end2,-Env.monomerDiameter.getValue()/2,uVec);
		end2MonCenter.copyToVector3d(end2Vec3d);
		end2T3D.setTranslation(end2Vec3d);
		end2TG.setTransform(end2T3D);
		
		coord.copyToVector3d(coordVec3d);
		coordT3D.setTranslation(coordVec3d);
		coordTG.setTransform(coordT3D);
		
		if (coordSysOn) {
			xLineEndPt.add(coord,0.5*length+coordLineLength,uVec);
			xLine.setCoordinate(0,coord);
			xLine.setCoordinate(1,xLineEndPt);
			yLineEndPt.add(coord,coordLineLength,yVec);
			yLine.setCoordinate(0,coord);
			yLine.setCoordinate(1,yLineEndPt);
			zLineEndPt.add(coord,coordLineLength,zVec);
			zLine.setCoordinate(0,coord);
			zLine.setCoordinate(1,zLineEndPt);
		} 
		
		cylT3D.setRotation(mxToX);
		cylT3D.mul(cylRot);
		cylT3D.setTranslation(coordVec3d);
		cylTG.setTransform(cylT3D);
	
		if (updateGraphicsLength) {
			G.removeChild(cylBG);
			makeNewCyl();
		
			updateGraphicsLength = false;
		}
		
	}
	
	public void addCoordSysGraphics () {
		if (!coordSysOn) {
			G.addChild(coordBG);
			G.addChild(coordSysG);
			coordSysOn = true;
		}
	}
	
	public void removeCoordSysGraphics () {
		if (coordSysOn) {
			coordBG.detach();
			coordSysG.detach();
			coordSysOn = false;
		}
	}

	
	
}


